#!/usr/bin/env perl
# 
# Copyright (c) 2009-2023 The TEBA Project. All rights reserved.
# 
# Redistribution and use in source, with or without modification, are
# permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# 
# Author: Atsuhi Yoshida


use warnings;
use strict;

use JSON;

use FindBin qw($Bin);
use lib "$Bin/../TEBA";

# DEBUG 用
#use Data::Dumper;

# オプションの処理
use Getopt::Std;
my %opts = ();
if (!getopts("abh", \%opts) || $opts{h}) {
    print STDERR "cfg2dot.pl [-ah] [json-based-cfg-file]\n",
	"  -a: generate AST graph with CFG graph.\n",
#	"  -b: add block links(from begin to end) in CFG.\n",
	"  -h: help.\n";
    exit(1);
}


use CFG;
# JSON 形式のデータを読み込み、抽象構文木(AST)を構成
my $root = JSON->new->decode(join('', <>));

my $cfg = CFG->load($root);

$cfg->with_ast_graph if $opts{a};
#$cfg->with_block_edge if $opts{b};

print $cfg->to_dot;
